package eu.olympus.client;

import eu.olympus.client.interfaces.ClientCryptoModule;
import eu.olympus.client.interfaces.CredentialManagement;
import eu.olympus.model.Policy;
import eu.olympus.model.PSCredential;
import eu.olympus.model.PresentationToken;
import eu.olympus.server.interfaces.PestoIdP;
import java.security.*;
import java.util.*;
import java.util.concurrent.Future;

import org.apache.commons.codec.Charsets;


public class PabcClient extends PestoAuthClient {

	private CredentialManagement credentialManagement;
	
	// Maybe instead of a list (and derive the integer identifier from the inferred order) a map with identifiers.
	public PabcClient(List<? extends PestoIdP> servers, CredentialManagement credentialManagement, ClientCryptoModule cryptoModule) throws NoSuchAlgorithmException {
		super(servers, cryptoModule);
		this.credentialManagement = credentialManagement;
	}

	@Override
	public String authenticate(String username, String password, Policy policy, String token, String type) {
		if(!credentialManagement.checkStoredCredential()){ //If no stored credential, get a new one
			try {
				long timestamp = System.currentTimeMillis();
				byte[][] signature = getSignedNonceAndUid(username, password, timestamp, PestoIdP.AUTHENTICATESTRING.getBytes(Charsets.UTF_8), token, type);
				Map<Integer,PSCredential> partialCredentials = new HashMap<>();
				Map<Integer,Future<String>> authentications = new HashMap<>();
				for (Integer i: servers.keySet()){
					authentications.put(i,executorService.submit(() -> servers.get(i).obtainCredential(username, cookies
							.get(i), timestamp, signature[i], timestamp)));
				}
				for(Integer iresp : authentications.keySet()) {
					PSCredential reconstructedCredential=new PSCredential(authentications.get(iresp).get());
					partialCredentials.put(iresp,reconstructedCredential);
				}
				PresentationToken generatedToken=credentialManagement.combineAndGeneratePresentationToken(partialCredentials, policy);
				return generatedToken!=null? generatedToken.getEncoded() : "Failed";
			} catch(Exception e) {
				return "Failed";
			}
		}
		return credentialManagement.generatePresentationToken(policy).getEncoded();
	}

	@Override
	public void clearSession() {
		super.clearSession();
		credentialManagement.clearCredential();
	}
}
