package eu.olympus.client;

import eu.olympus.server.interfaces.PestoIdP;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.concurrent.Future;

import org.apache.commons.codec.Charsets;
import eu.olympus.model.Policy;
import eu.olympus.model.exceptions.AuthenticationFailedException;
import eu.olympus.client.interfaces.ClientCryptoModule;
import eu.olympus.util.JWTUtil;

public class PestoClient extends PestoAuthClient {

	public PestoClient(List<? extends PestoIdP> servers, ClientCryptoModule cryptoModule) {
		super(servers, cryptoModule);
	}

	@Override
	public String authenticate(String username, String password, Policy policy, String token, String type) throws AuthenticationFailedException {
		try{
			long salt = System.currentTimeMillis();
			byte[][] signature = getSignedNonceAndUid(username, password, salt, PestoIdP.AUTHENTICATESTRING.getBytes(Charsets.UTF_8), token, type);
			List<String> partialTokens = new LinkedList<String>();
			List<Future<String>> authentications = new ArrayList<Future<String>>();
			for (PestoIdP server: servers.values()){
				authentications.add(executorService.submit(() -> server.authenticate(username, cookies.get(server.getId()), salt, signature[server.getId()], policy)));
			}
			for(Future<String> resp : authentications) {
				partialTokens.add(resp.get());
			}
			return JWTUtil.combineTokens(partialTokens, cryptoModule.getModulus());
		} catch(Exception e) {
			throw new AuthenticationFailedException(e);
		}
	}
}

