package eu.olympus.server.rest;

import com.google.protobuf.InvalidProtocolBufferException;
import eu.olympus.model.PasswordAuthentication;
import eu.olympus.model.exceptions.AuthenticationFailedException;
import eu.olympus.model.exceptions.UserCreationFailedException;
import eu.olympus.model.server.rest.AttributeMap;
import eu.olympus.model.server.rest.KeyAndCert;
import eu.olympus.model.server.rest.PasswordAuthenticationAndAttributes;
import eu.olympus.model.server.rest.PasswordAuthenticationAndCookie;
import eu.olympus.model.server.rest.PasswordAuthenticationAndIDProof;
import eu.olympus.model.server.rest.PasswordAuthenticationAndMFAToken;
import eu.olympus.model.server.rest.PasswordAuthenticationAndMFAType;
import eu.olympus.model.server.rest.PasswordAuthenticationAndPassword;
import eu.olympus.model.server.rest.PasswordAuthenticationAndPolicy;
import eu.olympus.server.PasswordJWTIdP;
import eu.olympus.util.KeySerializer;
import eu.olympus.util.keyManagement.CertificateUtil;
import eu.olympus.util.keyManagement.PemUtil;
import java.security.interfaces.RSAPrivateKey;
import javax.ws.rs.Consumes;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;
import org.apache.commons.codec.binary.Base64;

@Path("/idp")
public class PasswordIdPServlet{

	private PasswordJWTIdP idp = (PasswordJWTIdP)RESTIdPServer.getInstance().getIdP();
	
	@Path("setup")
	@POST
	@Consumes(MediaType.APPLICATION_JSON)
	public void setup(KeyAndCert keyPair) throws Exception{
		idp.setup((RSAPrivateKey)KeySerializer.deSerialize(keyPair.getPrivKey()),
				CertificateUtil.decodePemCert(keyPair.getCertificate()));
	}

	@Path("getcertificate")
	@GET
	@Produces(MediaType.APPLICATION_JSON)
	public String getCertificate() throws Exception {
			return PemUtil.encodeDerToPem(idp.getCertificate().getEncoded(), "CERTIFICATE");
	}


	@Path("createuser")
	@POST
	@Consumes(MediaType.APPLICATION_JSON)
	public void createUser(PasswordAuthentication creationData) throws UserCreationFailedException {
		idp.createUser(creationData);
	}

	@Path("prove")
	@POST
	@Consumes(MediaType.APPLICATION_JSON)
	public void proveId(PasswordAuthenticationAndIDProof creationData) throws UserCreationFailedException, InvalidProtocolBufferException {
		idp.proveId((PasswordAuthentication)creationData.getPasswordAuthentication(), Base64.decodeBase64(creationData.getCookie()), creationData.getIdentityProof());
	}
	
	
	@Path("createuserandprove")
	@POST	
	public void createUserAndProveId(PasswordAuthenticationAndIDProof creationData)
			throws UserCreationFailedException, InvalidProtocolBufferException {
		idp.createUserAndProveId(creationData.getPasswordAuthentication(), creationData.getIdentityProof());

	}

	@Path("authenticate")
	@POST
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	public String authenticate(PasswordAuthenticationAndPolicy authenticationData) throws Exception {
		return idp.authenticate((PasswordAuthentication)authenticationData.getPasswordAuthentication(), Base64.decodeBase64(authenticationData.getCookie()), authenticationData.getPolicy());
	}
	
	@Path("deleteAttribute")
	@POST
	@Consumes(MediaType.APPLICATION_JSON)
	public Boolean deleteAttribute(PasswordAuthenticationAndAttributes authenticationData) throws Exception {
		return idp.deleteAttribute((PasswordAuthentication)authenticationData.getPasswordAuthentication(), Base64.decodeBase64(authenticationData.getCookie()), authenticationData.getAttributes());
	}
	
	@Path("deleteAccount")
	@POST
	@Consumes(MediaType.APPLICATION_JSON)
	public Boolean deleteAccount(PasswordAuthenticationAndCookie authenticationData) throws Exception {
		return idp.deleteAccount(authenticationData.getPasswordAuthentication(), Base64.decodeBase64(authenticationData.getCookie()));
	}
	
	@Path("getAllAttributes")
	@POST
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	public AttributeMap getAllAttributes(PasswordAuthenticationAndCookie creationData) throws AuthenticationFailedException {
		return new AttributeMap(idp.getAllAttributes(creationData.getPasswordAuthentication(), Base64.decodeBase64(creationData.getCookie())));
	}
	
	@Path("changePassword")
	@POST
	@Consumes(MediaType.APPLICATION_JSON)
	public void changePassword(PasswordAuthenticationAndPassword passwordData) throws UserCreationFailedException, AuthenticationFailedException {
		idp.changePassword(passwordData.getPasswordAuthentication(), new PasswordAuthentication(passwordData.getPasswordAuthentication().getUsername(), passwordData.getNewPassword()), Base64.decodeBase64(passwordData.getCookie()));
	}
		
	@Path("requestMFA")
	@POST
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	public String requestMFA(PasswordAuthenticationAndMFAType request) throws AuthenticationFailedException {
		return idp.requestMFA(request.getPasswordAuthentication(), request.getType());
	}
	
	@Path("confirmMFA")
	@POST
	@Consumes(MediaType.APPLICATION_JSON)
	public boolean confirmMFA(PasswordAuthenticationAndMFAToken request) throws AuthenticationFailedException {
		return idp.confirmMFA(request.getPasswordAuthentication(), request.getToken(), request.getType());
	}

	@Path("removeMFA")
	@POST
	@Consumes(MediaType.APPLICATION_JSON)
	public boolean removeMFA(PasswordAuthenticationAndMFAToken request) throws AuthenticationFailedException {
		return idp.removeMFA(request.getPasswordAuthentication(), request.getToken(), request.getType());
	}

	@Path("generateSession")
	@POST
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	public String generateSession(PasswordAuthenticationAndMFAToken request) throws AuthenticationFailedException {
		return idp.generateSession(request.getPasswordAuthentication(), request.getToken(), request.getType());
	}
}