package eu.olympus.unit.server;

import eu.olympus.model.PasswordAuthentication;
import eu.olympus.model.Policy;
import eu.olympus.model.exceptions.AuthenticationFailedException;
import eu.olympus.server.AbstractPasswordIdP;
import eu.olympus.server.PasswordHandler;
import eu.olympus.server.storage.InMemoryKeyDB;
import eu.olympus.server.storage.InMemoryUserPasswordDatabase;

import java.security.cert.Certificate;
import java.util.Arrays;
import org.junit.Test;

public class TestAbstractPasswordIdP {


	
	@Test(expected = AuthenticationFailedException.class)
	public void testChangePasswordFailingWithBadCookie() throws Exception {
		InMemoryUserPasswordDatabase db = new InMemoryUserPasswordDatabase();
		db.addUser("user", "salt", "password");
		AbstractPasswordIdP idp = new TestIdP(new PasswordHandler(db, null, new InMemoryKeyDB(), null));
		idp.changePassword(new PasswordAuthentication("user", "bad_password"), new PasswordAuthentication("user", "password2"), "cookie".getBytes());
	}
	
	@Test(expected = AuthenticationFailedException.class)
	public void testRequestMFANullPointer() throws Exception {
		InMemoryUserPasswordDatabase db = new InMemoryUserPasswordDatabase();
		db.addUser("user", "salt", "password");
		AbstractPasswordIdP idp = new TestIdP(new PasswordHandler(db, null, new InMemoryKeyDB(), null));
		idp.requestMFA(new PasswordAuthentication("user", "password"), null);
	}
	
	@Test(expected = AuthenticationFailedException.class)
	public void testGetAllAttributesFailingWithBadCookie() throws Exception {
		InMemoryUserPasswordDatabase db = new InMemoryUserPasswordDatabase();
		db.addUser("user", "salt", "password");
		AbstractPasswordIdP idp = new TestIdP(new PasswordHandler(db, null, new InMemoryKeyDB(), null));
		idp.getAllAttributes(new PasswordAuthentication("user", "bad_password"), "cookie".getBytes());
	}
	
	@Test(expected = AuthenticationFailedException.class)
	public void testDeleteAttributeFailingWithBadCookie() throws Exception {
		InMemoryUserPasswordDatabase db = new InMemoryUserPasswordDatabase();
		db.addUser("user", "salt", "password");
		AbstractPasswordIdP idp = new TestIdP(new PasswordHandler(db, null, new InMemoryKeyDB(), null));
		idp.deleteAttribute(new PasswordAuthentication("user", "bad_password"), "cookie".getBytes(), Arrays.asList("item"));
	}
	
	private class TestIdP extends AbstractPasswordIdP {

		public TestIdP(PasswordHandler handler) {
			this.authenticationHandler = handler;
		}
		
		@Override
		public Certificate getCertificate() {
			// TODO Auto-generated method stub
			return null;
		}

		@Override
		public int getId() {
			// TODO Auto-generated method stub
			return 0;
		}

		@Override
		public String authenticate(PasswordAuthentication authenticationData, byte[] cookie, Policy policy)
				throws AuthenticationFailedException {
			// TODO Auto-generated method stub
			return null;
		}
		
	}
}
