package eu.olympus.client.rest;

import javax.servlet.ServletContext;
import javax.ws.rs.Consumes;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.MediaType;

import eu.olympus.client.interfaces.UserClient;

import eu.olympus.client.rest.model.PasswordAuthenticationAndIDProof;
import eu.olympus.model.PasswordAuthentication;
import eu.olympus.model.server.rest.AttributeMap;
import eu.olympus.model.server.rest.PasswordAuthenticationAndAttributes;
import eu.olympus.model.server.rest.PasswordAuthenticationAndPassword;
import eu.olympus.model.server.rest.PasswordAuthenticationAndPolicy;

@Path("/user")
public class UserClientServlet {
	
	@Context ServletContext context;
	
	@Path("createUser")
	@POST
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	public void createUser(PasswordAuthentication request) throws Exception {
		System.out.println("Got a createUser request");
		UserClient user = (UserClient) context.getAttribute("user");
		user.createUser(request.getUsername(), request.getPassword());
		System.out.println("Finished creating user");
	}
	
	@Path("createUserAndProveIdentity")
	@POST
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	public void createUserAndProveIdentity(PasswordAuthenticationAndIDProof request) throws Exception {
		System.out.println("Got a createUserAndProveIdentity request");
		UserClient user = (UserClient) context.getAttribute("user");
		PasswordAuthentication rq = request.getPasswordAuthentication();
		user.createUserAndProveIdentity(rq.getUsername(), rq.getPassword(), request.getIdentityProof());
		System.out.println("Finished creating user and proving identity");
	}
	
	@Path("proveIdentity")
	@POST
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	public void proveIdentity(PasswordAuthenticationAndIDProof request) throws Exception {
		System.out.println("Got a proveIdentity request");
		UserClient user = (UserClient) context.getAttribute("user");
		PasswordAuthentication rq = request.getPasswordAuthentication();
		user.proveIdentity(rq.getUsername(), rq.getPassword(), request.getIdentityProof(), null, "NONE");
		System.out.println("Finished proving identity");
	}
	
	
	@Path("authenticate")
	@POST
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	public String authenticate(PasswordAuthenticationAndPolicy request) throws Exception {
		System.out.println("Got an authenticate request");
		UserClient user = (UserClient) context.getAttribute("user");
		PasswordAuthentication rq = request.getPasswordAuthentication();
		return user.authenticate(rq.getUsername(), rq.getPassword(), request.getPolicy(), null, "NONE");
	}

	@Path("getAllAttributes")
	@POST
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	public AttributeMap getAllAttributes(PasswordAuthentication request) throws Exception {
		System.out.println("Got an getAllAttributes request");
		UserClient user = (UserClient) context.getAttribute("user");
		return new AttributeMap(user.getAllAttributes(request.getUsername(), request.getPassword(), null, "NONE"));
	}

	@Path("deleteAttributes")
	@POST
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	public void deleteAttributes(PasswordAuthenticationAndAttributes request) throws Exception {
		System.out.println("Got an deleteAttributes request");
		UserClient user = (UserClient) context.getAttribute("user");
		user.deleteAttributes(request.getPasswordAuthentication().getUsername(),
				request.getPasswordAuthentication().getPassword(), request.getAttributes(), null, "NONE");
		System.out.println("Attributes deleted");
	}
	

	@Path("deleteAccount")
	@POST
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	public void deleteAccount(PasswordAuthentication request) throws Exception {
		System.out.println("Got an deleteAccount request");
		UserClient user = (UserClient) context.getAttribute("user");
		user.deleteAccount(request.getUsername(), request.getPassword(), null, "NONE");
		System.out.println("Account deleted");
	}
	
	@Path("changePassword")
	@POST
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	public void changePassword(PasswordAuthenticationAndPassword request) throws Exception {
		System.out.println("Got an deleteAccount request");
		UserClient user = (UserClient) context.getAttribute("user");
		user.changePassword(request.getPasswordAuthentication().getUsername(), request.getPasswordAuthentication().getPassword(), request.getNewPassword(), null, "NONE");
		System.out.println("Account deleted");
	}
}
